var path = require('path');
var ExtractTextPlugin = require("extract-text-webpack-plugin");
var RawSource = require("webpack-sources").RawSource;
var css;

//https://lihuanghe.github.io/assets/images/webpack-module.png
//http://gold.xitu.io/entry/576b7aeda633bd0064065c74

function VHtmlWebpackPlugin(option) {
    this.option = option || {};
    css = new ExtractTextPlugin('[name]_[contenthash:8].css')
}

VHtmlWebpackPlugin.prototype.apply
    = function (compiler) {
    var me = this,
        publicPath = compiler.options.output.publicPath || '';

    css.apply(compiler);
    compiler.plugin('after-compile', function (compilation, callback) {
        var cssFileMap = {}, chunkHashMap = {}, chunkParentMap = {};

        // 登记chunk依赖
        compilation.chunks.forEach(function (chunk) {
            chunkHashMap[chunk.name] = chunk.hash;
            if (!chunk.entry && chunk.parents && chunk.parents.length) {
                chunk.parents.forEach(function (parentChunk) {
                    if (chunkParentMap[chunk.name]) {
                        chunkParentMap[chunk.name].push({name: parentChunk.name, hash: parentChunk.hash});
                    } else {
                        chunkParentMap[chunk.name] = [{name: parentChunk.name, hash: parentChunk.hash}]
                    }
                })
            }
        });


        // 重命名ExtractTextPlugin导出的css
        for (var key in compilation.assets) {
            if (/css$/.test(key)) {
                var m = key.split('.html_'),
                    cssPath = me.option.cssReleasePath ? (me.option.cssReleasePath + m[0].split('/')[m[0].split('/').length - 1] + '_' + m[1]) : (m[0] + '_' + m[1]);

                cssFileMap[m[0] + '.html'] = cssPath;
                compilation.assets[cssPath] = compilation.assets[key];
                delete compilation.assets[key];
            }
        }

        // 导出chunk中的html
        // 导出html对于的js
        for (var key in compilation.assets) {

            if (/html$/.test(key)) {
                var codeTpl, codeTrunk, assets = [];
                var jsChunkKey = (me.option.jsReleasePath ? me.option.jsReleasePath + key.replace(/\.html$/, '_').split('/')[key.split('/').length - 1] : key.replace(/\.html$/, '_')) + chunkHashMap[key].slice(0, 8) + '.js';
                codeTpl = codeTrunk = compilation.assets[key].source();
                codeTpl = codeTpl.replace(/<script [^>]*?src=[^>]*?(__webpack_require__\(\d+\))[^<]*?<\/script>/g, function (m, r) {
                    assets.push(r);
                    return '';
                });
                codeTpl = codeTpl.replace(/<link [^>]*?href=[^>]*?(__webpack_require__\(\d+\))[^>]*?>/g, function (m, r) {
                    assets.push(r);
                    return '';
                });
                if (assets.length) {
                    codeTrunk = codeTrunk.replace(/(\/\* 0 \*\/|\/\*\*\*\/ 0:)[\s\S]*?(\/\*\*\*\/ \})/, function (m, $1) {
                        var assetsStr = assets.join('+') || '""';
                        return $1 + '\n/***/ function(module, exports, __webpack_require__) { \n\tmodule.exports = ' + assetsStr + ';\n/***/}';
                    });
                    // 导出html对于的js
                    compilation.assets[jsChunkKey] = new RawSource(codeTrunk);
                }


                var codeTplStyle = ['<link rel=\\\"stylesheet\\\" href=\\\"' + publicPath + cssFileMap[key] + '\\\"/></head>'];
                var codeTplScript = ['<script src=\\\"' + publicPath + jsChunkKey + '\\\"></script></body>'];
                var codeTplOutput = [];
                if (chunkParentMap[key]) {
                    codeTplOutput.push('var window = {};');
                    chunkParentMap[key].forEach(function (parentChunk) {
                        var commonChunkKey = me.option.commonsChunkReleasePath + parentChunk.name + '_' + parentChunk.hash.slice(0, 8) + '.js';
                        codeTplOutput.push(compilation.assets[parentChunk.name].source());
                        codeTplScript.unshift('<script src=\\\"' + publicPath + commonChunkKey + '\\\"></script>');
                        if (!compilation.assets[commonChunkKey]) {
                            // 重命名commonChunk
                            compilation.assets[commonChunkKey] = compilation.assets[parentChunk.name];
                        }
                    });
                    codeTplOutput.push('var webpackJsonp = window.webpackJsonp;');
                }
                codeTpl = codeTpl.replace('</head>', codeTplStyle.join(''));
                codeTpl = codeTpl.replace('</body>', codeTplScript.join(''));
                codeTplOutput.push(codeTpl);
                // 导出chunk中的html
                compilation.assets[key] = new RawSource(eval(codeTplOutput.join('')));
            }
        }

        callback();
    });
};

module.exports = VHtmlWebpackPlugin;
module.exports.extract = ExtractTextPlugin.extract;